package net.wanji.opt.controllerv2;

import cn.hutool.core.date.DateUtil;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.opt.servicev2.CrossService;
import net.wanji.opt.synthesis.pojo.CrossRealTimeAlarmEntity;
import net.wanji.opt.synthesis.pojo.CrossStatusDisOptTimeEntity;
import net.wanji.opt.synthesis.pojo.vo.CrossOrGreenWaveTypeEntity;
import net.wanji.opt.vo2.CrossBaseInfoVO;
import net.wanji.opt.vo2.CrossRealTimeAlarmVO;
import net.wanji.opt.vo2.CrossStatusDistributionVO;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author zhengyifan
 * @date 2025/3/15
 */
@Api(value = "CrossController", description = "监测详情-路口事件详情")
@RequestMapping(value = "/cross")
@RestController
@Slf4j
public class CrossController {
    @Resource
    private CrossService crossService;

    @GetMapping("/getCrossStatusDistribution")
    @ApiOperation(httpMethod="GET",value="路口状态分布", notes="路口状态分布")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossID", value = "路口ID", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "groupType", value = "时间粒度 0--5分钟 1--15分钟 2--30分钟 3--60分钟", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "date", value = "日期 格式：yyyy-MM-dd", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "objectType", value = "范围 1：方向级指标 2：转向级指标 3：车道级指标 4：路口级指标", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "condition", value = "筛选条件", dataType = "String", paramType = "query"),
    })

    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossStatusDistributionVO.class),
    })
    public JsonViewObject getCrossStatusDistribution(String crossID, String date, String groupType, Integer objectType, String condition) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            if (StringUtils.isBlank(date)) {
                date = DateUtil.now().substring(0, 10);
            }
            Map<String, Object> result = crossService.getCrossStatusDistribution(crossID, date, groupType, objectType, condition);
            List<CrossStatusDisOptTimeEntity> optTimes = crossService.getOptTimeList(crossID);
            List<CrossOrGreenWaveTypeEntity> typeList = crossService.getCrossTypeList(crossID, date);
            result.put("optTimesList", optTimes);
            result.put("typeList", typeList);
            return jsonViewObject.success(result);
        } catch (Exception e) {
            log.error("监测详情-路口事件详情-路口状态分布: ", e);
            return jsonViewObject.success();
        }
    }

    @ApiOperation(value = "路口实时告警", notes = "路口实时告警", response = JsonViewObject.class, httpMethod="GET")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossID", value = "路口ID", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "time", value = "时间 格式：yyyy-MM-dd HH:ii:ss", dataType = "String", paramType = "query"),
    })
    @GetMapping(value = "/getCrossRealTimeAlarm")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossRealTimeAlarmVO.class),
    })
    public JsonViewObject getCrossRealTimeAlarm(String crossID, @RequestParam(defaultValue = "") String time) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        List<CrossRealTimeAlarmEntity> result;
        try {
            result = crossService.getCrossRealTimeAlarm(crossID, time);
        } catch (Exception e) {
            log.error("监测详情-路口事件详情-路口实时告警: ", e);
            return jsonViewObject.fail("路口实时告警查询失败");
        }
        return jsonViewObject.success(result);
    }

    @ApiOperation(value = "路口基础信息", notes = "路口基础信息", response = JsonViewObject.class, httpMethod = "GET")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossID", value = "路口ID", required = true, dataType = "String", paramType = "query"),
    })
    @GetMapping(value = "/getCrossBaseInfo")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossBaseInfoVO.class)
    })
    public JsonViewObject getCrossBaseInfo(String crossID) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        CrossBaseInfoVO result;
        try {
            result = crossService.getCrossBaseInfos(crossID);
        } catch (Exception e) {
            log.error("监测详情-路口事件详情-路口基础信息: ", e);
            return jsonViewObject.fail("路口基础信息查询失败");
        }
        return jsonViewObject.success(result);
    }
}
