package net.wanji.opt.controllerv2;

import cn.hutool.core.date.DateUtil;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.vo.LightsStatusVO2;
import net.wanji.opt.servicev2.TrendServiceV2;
import net.wanji.opt.vo2.*;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2025/03/06 18:33
 */
@Api(value = "TrendControllerV2", description = "态势监测v2", tags = "态势监测v2")
@RequestMapping("/trendV2")
@RestController
@Slf4j
public class TrendControllerV2 {

    @Resource
    private TrendServiceV2 trendServiceV2;

    @ApiOperation(value = "通用接口获取服务器时间", notes = "通用接口获取服务器时间", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/getNow")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = String.class),
    })
    public JsonViewObject getNow() {
        return JsonViewObject.newInstance().success(DateUtil.now(), "");
    }

    @ApiOperation(value = "态势监测-区域体检-雷达图", notes = "态势监测-区域体检-雷达图", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossGreenStatusTimeRate")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossGreenStatusTimeRateVO.class),
    })
    public JsonViewObject crossGreenStatusTimeRate() {
        List<CrossGreenStatusTimeRateVO> list = Collections.emptyList();
        try {
            list = trendServiceV2.crossGreenStatusTimeRate();
        } catch (Exception e) {
            log.error("态势监测-区域体检-雷达图-查询失败：{}", e);
            JsonViewObject.newInstance().success(list);
        }
        return JsonViewObject.newInstance().success(list);
    }

    @ApiOperation(value = "态势监测-区域体检-路口列表", notes = "态势监测-区域体检-路口列表",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossOptInfoList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossOptInfoVO.class),
    })
    public JsonViewObject crossOptInfoList() throws Exception {
        List<CrossOptInfoVO> list = Collections.emptyList();
        try {
            list = trendServiceV2.crossOptInfoList();
        } catch (Exception e) {
            log.error("态势监测-区域体检-路口列表：{}", e);
            JsonViewObject.newInstance().success(list);
        }
        return JsonViewObject.newInstance().success(list);
    }

    @ApiOperation(value = "态势监测-区域体检-绿波列表", notes = "态势监测-区域体检-绿波列表",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/greenOptInfoList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenOptInfoVO.class),
    })
    public JsonViewObject greenOptInfoList() throws Exception {
        List<GreenOptInfoVO> list = Collections.emptyList();
        try {
            list = trendServiceV2.greenOptInfoList();
        } catch (Exception e) {
            log.error("态势监测-区域体检-绿波列表", e);
            JsonViewObject.newInstance().success(list);
        }
        return JsonViewObject.newInstance().success(list);
    }

    @ApiOperation(value = "态势监测-运行监测-干线优化监测", notes = "态势监测-运行监测-干线优化监测",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/greenOptMonitoringList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptMonitoringResultVO.class),
    })
    public JsonViewObject greenOptMonitoringList() throws Exception {
        OptMonitoringResultVO result = new OptMonitoringResultVO();
        try {
            List<OptMonitoringVO> list = trendServiceV2.greenOptMonitoringList();
            countResult(result, list);
        } catch (Exception e) {
            log.error("态势监测-运行监测-干线优化监测-查询异常：", e);
            JsonViewObject.newInstance().success(result);
        }
        return JsonViewObject.newInstance().success(result);
    }

    private void countResult(OptMonitoringResultVO result, List<OptMonitoringVO> list) throws Exception {
        Map<String, List<OptMonitoringVO>> listMap = list.stream().collect(Collectors.groupingBy(OptMonitoringVO::getType));
        List<OptMonitoringResultVO.TypeCount> typeCounts = new ArrayList<>();
        for (Map.Entry<String, List<OptMonitoringVO>> entry : listMap.entrySet()) {
            String key = entry.getKey();
            List<OptMonitoringVO> value = entry.getValue();
            OptMonitoringResultVO.TypeCount typeCount = new OptMonitoringResultVO.TypeCount();
            typeCount.setType(key);
            typeCount.setEventCount(value.size());
            List<OptMonitoringVO> optList = value.stream().filter(vo -> Objects.equals(1, vo.getOptStatus())).collect(Collectors.toList());
            typeCount.setOptCount(optList.size());
            typeCounts.add(typeCount);
        }

        result.setTypeCount(typeCounts);
        result.setOptMonitoringVOList(list);
    }

    @ApiOperation(value = "态势监测-运行监测-路口优化监测", notes = "态势监测-运行监测-路口优化监测",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossOptMonitoringList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptMonitoringResultVO.class),
    })
    public JsonViewObject crossOptMonitoringList() throws Exception {
        OptMonitoringResultVO result = new OptMonitoringResultVO();
        try {
            List<OptMonitoringVO> list = trendServiceV2.crossOptMonitoringList();
            countResult(result, list);
        } catch (Exception e) {
            log.error("态势监测-运行监测-路口优化监测：", e);
            JsonViewObject.newInstance().success(result);
        }
        return JsonViewObject.newInstance().success(result);
    }

    @ApiOperation(value = "态势监测-事件告警-实时列表", notes = "态势监测-事件告警-实时列表",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/eventAlarmRealTimeList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptMonitoringVO.class),
    })
    public JsonViewObject eventAlarmRealTimeList(String type) throws Exception {
        List<OptMonitoringVO> list = Collections.emptyList();
        try {
            if (StringUtils.isBlank(type)) {
                list = trendServiceV2.eventAlarmRealTimeList();
            } else {
                list = trendServiceV2.eventAlarmRealTimeList().stream()
                       .filter(vo -> StringUtils.equalsIgnoreCase(type, vo.getType()))
                       .collect(Collectors.toList());
            }
        } catch (Exception e) {
            log.error("态势监测-事件告警-实时列表：", e);
            JsonViewObject.newInstance().success(list);
        }
        List<OptMonitoringVO> collect = list.stream().
                filter(vo -> !StringUtils.equalsIgnoreCase("700", vo.getType())
                && !StringUtils.equalsIgnoreCase("708", vo.getType())).collect(Collectors.toList());
        return JsonViewObject.newInstance().success(collect);
    }

    @ApiOperation(value = "态势监测-区域体检-雷达图绿波类型过滤", notes = "态势监测-区域体检-雷达图绿波类型过滤",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/selectGreenEventTypeCountTimeList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptMonitoringVO.class),
    })
    public JsonViewObject selectGreenEventTypeCountTimeList(@RequestBody EventTypeCountTimeVO vo) throws Exception {
        List<StatisticsEventTypeCountTimeVO> list = Collections.emptyList();
        try {
            list = trendServiceV2.selectGreenEventTypeCountTimeList(vo);
        } catch (Exception e) {
            log.error("态势监测-区域体检-雷达图绿波类型过滤：", e);
            JsonViewObject.newInstance().success(list);
        }
        return JsonViewObject.newInstance().success(list);
    }

    @ApiOperation(value = "态势监测-区域体检-雷达图路口类型过滤", notes = "态势监测-区域体检-雷达图路口类型过滤",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/selectCrossEventTypeCountTimeList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptMonitoringVO.class),
    })
    public JsonViewObject selectCrossEventTypeCountTimeList(@RequestBody EventTypeCountTimeVO vo) throws Exception {
        List<StatisticsEventTypeCountTimeVO> list = Collections.emptyList();
        try {
            list = trendServiceV2.selectCrossEventTypeCountTimeList(vo);
        } catch (Exception e) {
            log.error("态势监测-区域体检-雷达图路口类型过滤：", e);
            JsonViewObject.newInstance().success(list);
        }
        return JsonViewObject.newInstance().success(list);
    }

    @ApiOperation(value = "态势监测-事件告警-历史列表", notes = "态势监测-事件告警-历史列表",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/eventAlarmHistList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptMonitoringVO.class),
    })
    public JsonViewObject eventAlarmHistList(int pageNo, int pageSize) throws Exception {
        PageInfo<StatisticsEventTypeCountTimeVO> results = new PageInfo<>();
        try {
            results = trendServiceV2.selectCrossGreenHistList(pageNo, pageSize);
        } catch (Exception e) {
            log.error("态势监测-事件告警-历史列表：", e);
            JsonViewObject.newInstance().success(results);
        }
        return JsonViewObject.newInstance().success(results);
    }

    @ApiOperation(value = "态势监测-策略推荐-优化策略", notes = "态势监测-策略推荐-优化策略",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/optStrategyResultInfo")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = String.class),
    })
    public JsonViewObject optStrategyResultInfo(String id, String date) throws Exception {
        OptStrategyResultInfoVO result = new OptStrategyResultInfoVO();
        try {
            result = trendServiceV2.optStrategyResultInfo(id, date);
        } catch (Exception e) {
            log.error("态势监测-策略推荐-优化策略：", e);
            JsonViewObject.newInstance().success(result, "");
        }
        return JsonViewObject.newInstance().success(result, "");
    }

    @ApiOperation(value = "态势监测-策略推荐-绿波优化相位差", notes = "态势监测-策略推荐-绿波优化相位差",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/optStrategyCrossOffsetList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptMonitoringVO.class),
    })
    public JsonViewObject optStrategyCrossOffsetList(Integer id, String date) throws Exception {
        GreenOptCrossOffsetVO greenOptCrossOffsetVO = new GreenOptCrossOffsetVO();
        try {
            greenOptCrossOffsetVO = trendServiceV2.optStrategyCrossOffsetList(id, date);
        } catch (Exception e) {
            log.error("态势监测-策略推荐-绿波优化相位差：", e);
            JsonViewObject.newInstance().success(greenOptCrossOffsetVO);
        }
        return JsonViewObject.newInstance().success(greenOptCrossOffsetVO);
    }


    @ApiOperation(value = "态势监测-查询历史灯态", notes = "态势监测-查询历史灯态",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/selectLightStatusHist")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO2.class),
    })
    public JsonViewObject selectLightStatusHist(String id, String date) throws Exception {
        LightsStatusVO2 lightsStatusVO2 = null;
        try {
            lightsStatusVO2 = trendServiceV2.selectLightStatusHist(id, date);
        } catch (Exception e) {
            log.error("态势监测-查询历史灯态：", e);
            JsonViewObject.newInstance().success(lightsStatusVO2);
        }
        return JsonViewObject.newInstance().success(lightsStatusVO2);
    }
}