package net.wanji.opt.controllerv2.comprehensivequery;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.entity.comprehensivequery.CrossLaneDataHistPoExtendName;
import net.wanji.opt.entity.comprehensivequery.GreenTreeEntity;
import net.wanji.opt.servicev2.comprehensivequery.ComprehensiveQueryService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;
import java.util.ArrayList;
import java.util.List;

/**
 * @author huangwm
 * @date 2025/03/06 18:33
 */
@Api(value = "ComprehensiveQueryController", description = "综合查询", tags = "综合查询")
@RequestMapping("/ComprehensiveQuery")
@RestController
@Slf4j
public class ComprehensiveQueryController {
    @Resource
    private ComprehensiveQueryService comprehensiveQueryService;

    @ApiOperation(value = "获取干线与路口树", notes = "获取干线与路口树", httpMethod="GET", response =JsonViewObject.class,
                 produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/getGreenAndCross")
    @ApiImplicitParams({
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = String.class),
    })
    public JsonViewObject getGreenAndCross() {
        List<GreenTreeEntity> list = new ArrayList<>();
        try {
            list = comprehensiveQueryService.getGreenAndCross();
        } catch (Exception e) {
            log.error("综合查询-获取干线与路口树-查询失败：{}", e);
            JsonViewObject.newInstance().success(list);
        }
        return JsonViewObject.newInstance().success(list);
    }

    @ApiOperation(value = "获取综合查询数据", notes = "获取综合查询数据", httpMethod="GET", response =JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/getComprehensiveQuery")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口id", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "日期 格式：yyyy-MM-dd h:i:s", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "日期 格式：yyyy-MM-dd h:i:s", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "objectType", value = "范围 0：路口级指标 1：方向级指标 2：转向级指标", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "groupType", value = "时间粒度 0--5分钟 1--15分钟 2--30分钟 3--60分钟", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageNum", value = "分页页码", required = true, dataType = "Integer", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页数据条数", required = true, dataType = "Integer", paramType = "query"),
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = String.class),
    })
    public JsonViewObject getComprehensiveQuery(String crossId , String startTime ,String endTime ,String objectType ,String groupType,Integer pageNum,Integer pageSize) {

        try {
            JSONObject list  = comprehensiveQueryService.getCrossStatusDistribution(crossId, startTime , endTime , objectType , groupType,pageNum,pageSize);
            return JsonViewObject.newInstance().success(list);
        } catch (Exception e) {
            log.error("综合查询-综合查询数据-查询失败：{}", e);
            return JsonViewObject.newInstance().success("");
        }
    }
}
