package net.wanji.opt.controllerv2.eventoptimize;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.controllerv2.judgeanalysis.design.response.crossproblem.CrossProblemDistributeAnalysisResult;
import net.wanji.opt.entity.eventoptimize.TEventOptimizeInfo;
import net.wanji.opt.entity.eventoptimize.TEventOptimizeInfoVO;
import net.wanji.opt.servicev2.eventoptimize.TEventOptimizeInfoService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;

@RestController
@Slf4j
@RequestMapping("/event-optimize-info")
@Api(value="TEventOptimizeInfoController", description="报警优化信息接口", tags = "报警优化信息")
public class TEventOptimizeInfoController {

    @Resource
    private TEventOptimizeInfoService tEventOptimizeInfoService;

    // 新增接口用于调用带有条件的分页查询
    @ApiOperation(value = "报警优化信息-事件优化记录查询-根据条件分页查询记录", notes = "根据条件分页查询记录",  produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON,hidden = false)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "currentPage", value = "页数", required = true, dataType = "int",defaultValue = "1",example = "1"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", required = true, dataType = "int",defaultValue = "10"),
            @ApiImplicitParam(name = "startTime", value = "开始时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-03-21 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "结束时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-03-21 23:00:00"),
            @ApiImplicitParam(name = "crossId", value = "路口id", required = false, dataType = "String",defaultValue = "",example = "13NAF0B5PH0"),
            @ApiImplicitParam(name = "greenId", value = "干线id", required = false, dataType = "int",defaultValue = "",example = "4"),
    })
    @GetMapping("/selectPageWithCrossIdAndGreenId")
    public JsonViewObject selectPageWithCrossIdAndGreenId(
            @RequestParam int currentPage,
            @RequestParam int pageSize,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime,
            @RequestParam(required = false) Integer greenId,
            @RequestParam(required = false) String crossId) {

        JsonViewObject jsonView = JsonViewObject.newInstance();
        try {
            Page<TEventOptimizeInfoVO> retData = tEventOptimizeInfoService.selectPageWithCrossIdAndGreenId(currentPage, pageSize, startTime, endTime, crossId,greenId);
            jsonView.success(retData);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonView;

    }

    @ApiOperation(value = "报警优化信息-根据条件分页查询记录", notes = "根据条件分页查询记录",  produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON,hidden = false)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "currentPage", value = "页数", required = true, dataType = "int",defaultValue = "1",example = "1"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", required = true, dataType = "int",defaultValue = "10"),
            @ApiImplicitParam(name = "startTime", value = "开始时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-03-21 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "结束时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-03-21 23:00:00"),
            @ApiImplicitParam(name = "crossId", value = "路口id", required = true, dataType = "String",defaultValue = "",example = "13MOD0B5SI0"),
            @ApiImplicitParam(name = "eventType", value = "事件类型", required = false, dataType = "string", defaultValue = "1"),
            @ApiImplicitParam(name = "optStatus", value = "优化状态", required = false, dataType = "integer", defaultValue = "1"),
            @ApiImplicitParam(name = "duration", value = "持续时间", required = false, dataType = "integer", defaultValue = "1"),
            @ApiImplicitParam(name = "startOptDuration", value = "优化开始时间", required = false, dataType = "integer", defaultValue = "0"),
            @ApiImplicitParam(name = "endOptDuration", value = "优化结束时间", required = false, dataType = "integer", defaultValue = "300"),
    })
    @GetMapping("/pageWithConditions")
    public JsonViewObject queryPageWithConditions(
            @RequestParam int currentPage,
            @RequestParam int pageSize,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime,
            @RequestParam(required = false) String crossId,
            @RequestParam(required = false) String eventType,
            @RequestParam(required = false) Integer optStatus,
            @RequestParam(required = false) Integer duration,
            @RequestParam(required=false) Integer startOptDuration,
            @RequestParam(required=false) Integer endOptDuration) {

        JsonViewObject jsonView = JsonViewObject.newInstance();
        try {
            Page<TEventOptimizeInfo> retData = tEventOptimizeInfoService.queryPageWithConditions(currentPage, pageSize, startTime, endTime, crossId,eventType,optStatus,duration,startOptDuration,endOptDuration);
            jsonView.success(retData);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonView;

    }

    @ApiOperation(value = "路口事件数按事件数/持续时间分布情况", notes = "路口事件数按事件数/持续时间分布情况", httpMethod = "GET")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "greenId", value = "干线ID", required = false, dataType = "String"),
            @ApiImplicitParam(name = "crossId", value = "路口ID", required = false, dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "查询截止时间，格式yyyy-MM-dd HH:mm:ss", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "查询截止时间，格式yyyy-MM-dd HH:mm:ss", required = true, dataType = "String"),
            @ApiImplicitParam(name = "groupType", value = "时间粒度类型 0：5分钟粒度 1：15分钟粒度 2：30分钟粒度 3：小时粒度 4：天粒度", required = true, dataType = "Integer",defaultValue = "1"),
    })
    @GetMapping(value = "/getCrossOptimizeDistribute")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossProblemDistributeAnalysisResult.class),
    })
    public JsonViewObject getCrossOptimizeDistribute(
            @RequestParam(required = false) Integer greenId,
            @RequestParam(required = false) String crossId,
            @RequestParam String startTime,
            @RequestParam String endTime,
            @RequestParam Integer groupType) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            JSONObject result = tEventOptimizeInfoService.getCrossOptimizeDistribute(greenId,crossId,startTime,endTime,groupType);
            return jsonViewObject.success(result);
        } catch (Exception e) {
            log.error("路口事件数按事件数/持续时间分布情况", e);
            return jsonViewObject.fail("路口事件数按事件数/持续时间分布情况失败");
        }
    }

    // 新增绿波干线接口用于调用带有条件的分页查询
    @ApiOperation(value = "报警优化干线信息-根据条件分页查询记录", notes = "根据条件分页查询记录",  produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON,hidden = false)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "currentPage", value = "页数", required = true, dataType = "int",defaultValue = "1",example = "1"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", required = true, dataType = "int",defaultValue = "10"),
            @ApiImplicitParam(name = "startTime", value = "开始时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-03-21 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "结束时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-03-21 23:00:00"),
            @ApiImplicitParam(name = "greenId", value = "绿波干线", required = true, dataType = "integer", defaultValue = "1"),
            @ApiImplicitParam(name = "eventType", value = "事件类型", required = false, dataType = "string", defaultValue = "1"),
            @ApiImplicitParam(name = "optStatus", value = "优化状态", required = false, dataType = "integer", defaultValue = "1"),
            @ApiImplicitParam(name = "duration", value = "持续时间", required = false, dataType = "integer", defaultValue = "1"),
            @ApiImplicitParam(name = "startOptDuration", value = "优化开始时间", required = false, dataType = "integer", defaultValue = "1"),
            @ApiImplicitParam(name = "endOptDuration", value = "优化结束时间", required = false, dataType = "integer", defaultValue = "1"),
    })
    @GetMapping("/pageWithLineConditions")
    public JsonViewObject pageWithLineConditions(
            @RequestParam int currentPage,
            @RequestParam int pageSize,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime,
            @RequestParam(required = false) Integer greenId,
            @RequestParam(required = false) String eventType,
            @RequestParam(required = false) Integer optStatus,
            @RequestParam(required = false) Integer duration,
            @RequestParam(required=false) Integer startOptDuration,
            @RequestParam(required=false) Integer endOptDuration){
        JsonViewObject jsonView = JsonViewObject.newInstance();
          try {
              Page<TEventOptimizeInfo> retLineData=tEventOptimizeInfoService.pageWithLineConditions(currentPage, pageSize, startTime, endTime, greenId,eventType,optStatus,duration,startOptDuration,endOptDuration);
              jsonView.success(retLineData);
          }catch (Exception e){
              jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
              log.error("{} getAll error", this.getClass().getSimpleName(), e);
          }
        return  jsonView;

    }

}