package net.wanji.opt.controllerv2.report;

import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.po.BaseAreaInfoPO;
import net.wanji.opt.controllerv2.report.vo.AreaParamBody;
import net.wanji.opt.servicev2.report.BaseAreaInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 区域基础信息 接口API
 * </p>
 *  @version 1.0
 * @author wangtao
 * @Date 2025-03-18
 */
@RestController
@Slf4j
@RequestMapping("/base-area-info")
@Api(value="BaseAreaInfoController", description="区域基础信息接口", tags = "区域基础信息")
public class BaseAreaInfoController {

    @Autowired
    private BaseAreaInfoService baseAreaInfoService;



    @ApiOperation(httpMethod="GET",value="区域基础信息-根据区划类型查询对应集合数据", notes="")
    @GetMapping(value = "/byCondition")
    public JsonViewObject getListByType(@RequestParam(value = "type", required = false, defaultValue = "2") String type) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            List<BaseAreaInfoPO> list = baseAreaInfoService.selectByType(type);
            jsonViewObject.success(list);
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

    @ApiOperation(httpMethod="POST",value="区域基础信息-自定义区域", notes="")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "name", value = "区域ID", required = true, dataType = "String"),
//            @ApiImplicitParam(name = "wkt", value = "区域边界", required = true, dataType = "String", defaultValue = "117.049347926,36.654744674;117.066122093,36.654772445;117.0800809517208,36.65440636491661;117.07979265456723,36.65214909599633;117.07706941120558,36.63829789737139;117.07414217382905,36.63736765765722;117.0707410731053,36.636677059391445;117.06550839703021,36.63722151604908;117.06267677771574,36.63667912391616;117.0577271202223,36.63483777099411;117.05251580910459,36.63437969486462;117.04433347163972,36.63696466063367;117.04843387376732,36.637934238407;117.04968488845323,36.638408635754985;117.049347926,36.654744674;117.066122093,36.654772445;117.0800809517208,36.65440636491661;117.07979265456723,36.65214909599633;117.07706941120558,36.63829789737139;117.07414217382905,36.63736765765722;117.0707410731053,36.636677059391445;117.06550839703021,36.63722151604908;117.06267677771574,36.63667912391616;117.0577271202223,36.63483777099411;117.05251580910459,36.63437969486462;117.04433347163972,36.63696466063367;117.04843387376732,36.637934238407;117.04968488845323,36.638408635754985;117.049347926,36.654744674"),
//            @ApiImplicitParam(name = "type", value = "区划类型：1行政区划；2交警辖区；3商圈；4交通小区；5热点区域；6道路 99自定义区域", required = true, dataType = "Integer", defaultValue = "99"),
//    })
    @PostMapping(value = "/insertAreaInfo")
    public JsonViewObject insertAreaInfo(@RequestBody AreaParamBody areaParamBody) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {

            BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
            baseAreaInfoPO.setName(areaParamBody.getName());
            baseAreaInfoPO.setPolylines(areaParamBody.getWkt());
            baseAreaInfoPO.setType(areaParamBody.getType());
            baseAreaInfoService.insertAreaInfo(baseAreaInfoPO);
            jsonViewObject.success();
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

    @ApiOperation(httpMethod="GET",value="干线路口基础信息-根据区域查询对应集合数据", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "areaId", value = "区域ID", required = false, dataType = "Integer", defaultValue = "12"),
            @ApiImplicitParam(name = "wkt", value = "区域边界", required = false, dataType = "String", defaultValue = "117.049347926,36.654744674;117.066122093,36.654772445;117.0800809517208,36.65440636491661;117.07979265456723,36.65214909599633;117.07706941120558,36.63829789737139;117.07414217382905,36.63736765765722;117.0707410731053,36.636677059391445;117.06550839703021,36.63722151604908;117.06267677771574,36.63667912391616;117.0577271202223,36.63483777099411;117.05251580910459,36.63437969486462;117.04433347163972,36.63696466063367;117.04843387376732,36.637934238407;117.04968488845323,36.638408635754985;117.049347926,36.654744674;117.066122093,36.654772445;117.0800809517208,36.65440636491661;117.07979265456723,36.65214909599633;117.07706941120558,36.63829789737139;117.07414217382905,36.63736765765722;117.0707410731053,36.636677059391445;117.06550839703021,36.63722151604908;117.06267677771574,36.63667912391616;117.0577271202223,36.63483777099411;117.05251580910459,36.63437969486462;117.04433347163972,36.63696466063367;117.04843387376732,36.637934238407;117.04968488845323,36.638408635754985;117.049347926,36.654744674"),
            @ApiImplicitParam(name = "type", value = "查询类型,1-干线信息 2-路口信息", required = false, dataType = "Integer", defaultValue = "1"),
    })
    @GetMapping(value = "/getGreenByArea")
    public JsonViewObject getGreenByAre(@RequestParam(value = "areaId", required = false, defaultValue = "12") Integer areaId,
                                        @RequestParam(value = "wkt", required = false) String wkt,
                                        @RequestParam(value = "type", required = false) Integer type) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {

            JSONObject jsonObject = baseAreaInfoService.getGreenByWkt(areaId,wkt,type);

            jsonViewObject.success(jsonObject);
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

}
