package net.wanji.opt.controllerv2.report;

import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.po.BaseAreaInfoPO;
import net.wanji.opt.entity.report.GreenWaveCrossRidInfo;
import net.wanji.opt.servicev2.report.BaseAreaInfoService;
import net.wanji.opt.servicev2.report.GreenWaveCrossRidService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 区域基础信息 接口API
 * </p>
 *  @version 1.0
 * @author
 * @Date 2025-03-18
 */
@RestController
@Slf4j
@RequestMapping("/green-wave-cross-rid")
@Api(value="GreenWaveCrossRidController", description="干线分析报告接口", tags = "干线分析报告-周报")
public class GreenWaveCrossRidController {

    @Autowired
    private GreenWaveCrossRidService greenWaveCrossRidService;



    @ApiOperation(httpMethod="GET",value="1.1-干线拥堵信息", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "干线ID,多个id用','分隔", required = true, dataType = "String",defaultValue = "1,2"),
            @ApiImplicitParam(name = "dayType", value = "高峰类型 1早高峰 2晚高峰 3全天", required = true, dataType = "String",defaultValue = "1"),
            })
    @GetMapping(value = "/getGreenWeekDataList")
    public JsonViewObject getGreenWeekDataList(@RequestParam(value = "ids", required = false, defaultValue = "2") String ids,
                                               @RequestParam(value = "type", required = false, defaultValue = "2") String type
    ) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            JSONObject object = greenWaveCrossRidService.getGreenInfoList(ids);
            jsonViewObject.success(object);
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

    @ApiOperation(httpMethod="GET",value="1.1-干线基础信息", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "干线ID,多个id用','分隔", required = true, dataType = "String",defaultValue = "1,2"),
            })
    @GetMapping(value = "/getGreenInfoList")
    public JsonViewObject getGreenInfoList(@RequestParam(value = "ids", required = false, defaultValue = "2") String ids) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            JSONObject object = greenWaveCrossRidService.getGreenInfoList(ids);
            jsonViewObject.success(object);
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

}
