package net.wanji.opt.controllerv2.report;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.controllerv2.judgeanalysis.design.response.areaproblem.AreaCrossProblemAnalysisResult;
import net.wanji.opt.controllerv2.report.design.GreenReportOverviewAnalysisResult;
import net.wanji.opt.controllerv2.report.design.GreenReportSamePeriodCompareResult;
import net.wanji.opt.controllerv2.report.vo.GreenReportSamePeriodCompareData;
import net.wanji.opt.entity.report.GreenWaveWeekData;
import net.wanji.opt.servicev2.report.GreenWaveCrossRidService;
import net.wanji.opt.servicev2.report.GreenWaveWeekDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 干线概况 接口API
 * </p>
 *  @version 1.0
 * @author
 * @Date 2025-03-18
 */
@RestController
@Slf4j
@RequestMapping("/green-wave-overview")
@Api(value="GreenWaveCrossRidController", description="干线分析报告接口", tags = "干线分析报告-周报")
public class GreenWaveOverviewController {

    @Autowired
    private GreenWaveCrossRidService greenWaveCrossRidService;
    @Autowired
    private GreenWaveWeekDataService greenWaveWeekDataService;



    @ApiOperation(httpMethod="GET",value="2.1-干线概况-总体情况说明", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "干线ID,多个id用','分隔【为空时查询所有干线】", required = true, dataType = "String",defaultValue = "1,2"),
            @ApiImplicitParam(name = "year", value = "年份", required = true, dataType = "String",defaultValue = "2025"),
            @ApiImplicitParam(name = "week", value = "一年的第几周", required = true, dataType = "String",defaultValue = "13"),
            })
    @GetMapping(value = "/getOverallSituation")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenReportOverviewAnalysisResult.class),
    })
    public JsonViewObject getGreenInfoList(Integer year,Integer week, String ids) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            JSONObject jsonObject = greenWaveWeekDataService.getGreenInfoList( year, week, ids);
            jsonViewObject.success(jsonObject);

        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }



    @ApiOperation(httpMethod="GET",value="2.2-干线概况-工作日同期对比", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "干线ID,多个id用','分隔", required = true, dataType = "String",defaultValue = "1,2"),
            @ApiImplicitParam(name = "year", value = "年份", required = true, dataType = "String",defaultValue = "2025"),
            @ApiImplicitParam(name = "week", value = "一年的第几周", required = true, dataType = "String",defaultValue = "13"),
    })
    @GetMapping(value = "/getSamePeriodCompare")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenReportSamePeriodCompareResult.class),
    })
    public JsonViewObject getSamePeriodCompare(Integer year,Integer week, String ids) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            List<GreenReportSamePeriodCompareData> jsonObject = greenWaveWeekDataService.getSamePeriodCompare(year,week,ids);
            jsonViewObject.success(jsonObject);
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

}
