package net.wanji.opt.controllerv2.syslog;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.entity.syslog.SysOptimizeLog;
import net.wanji.opt.servicev2.syslog.SysLogServer;
import net.wanji.opt.vo2.CrossGreenStatusTimeRateVO;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;
import java.util.Collections;
import java.util.List;

/**
 * @author huangwm
 * @date 2025/03/06 18:33
 */
@Api(value = "sysLogController", description = "系统日志", tags = "系统日志")
@RequestMapping("/sysLog")
@RestController
@Slf4j
public class sysLogController {
    @Resource
    private SysLogServer sysLogServer;

    @ApiOperation(value = "获取系统优化日志", notes = "获取系统优化日志",httpMethod="GET", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/getOptimizeLog")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口id", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "日期 格式：yyyy-MM-dd h:i:s", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "日期 格式：yyyy-MM-dd h:i:s", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageNum", value = "分页页码", required = true, dataType = "Integer", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页数据条数", required = true, dataType = "Integer", paramType = "query"),
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = String.class),
    })
    public JsonViewObject getOptimizeLog(String crossId,String startTime,String endTime,Integer pageNum,Integer pageSize) {
        try {
            JSONObject list = sysLogServer.findSysOptimizeLog(crossId,startTime,endTime,pageNum,pageSize);
            return JsonViewObject.newInstance().success(list);
        } catch (Exception e) {
            log.error("系统日志-优化日志-查询失败：{}", e);
            return JsonViewObject.newInstance().success("");
        }
    }
}
