package net.wanji.opt.servicev2.eventoptimize.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.wanji.databus.po.EventOptimizeCountPo;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.dao.mapper.eventoptimize.TEventOptimizeInfoMapper;
import net.wanji.opt.entity.eventoptimize.TEventOptimizeInfo;
import net.wanji.opt.entity.eventoptimize.TEventOptimizeInfoVO;
import net.wanji.opt.servicev2.eventoptimize.TEventOptimizeInfoService;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class TEventOptimizeInfoServiceImpl extends ServiceImpl<TEventOptimizeInfoMapper, TEventOptimizeInfo> implements TEventOptimizeInfoService {

    @Autowired
    private TEventOptimizeInfoMapper tEventOptimizeInfoMapper;

    // 新增接口用于带有条件的分页查询
    @Override
    public Page<TEventOptimizeInfo> queryPageWithConditions(int currentPage, int pageSize, String startTime, String endTime, String crossId,String eventType,Integer optStatus,Integer duration,Integer startOptDuration, Integer endOptDuration) {
        Page<TEventOptimizeInfo> page = new Page<>(currentPage, pageSize);
        Page<TEventOptimizeInfo> tEventOptimizeInfoPage = tEventOptimizeInfoMapper.selectPageWithConditions(page, startTime, endTime, crossId, eventType,optStatus,duration, startOptDuration,  endOptDuration);
        tEventOptimizeInfoPage.getRecords().forEach(x->{
            x.setEventLabal(getEventLabel(x.getEventType()));
        });
        return tEventOptimizeInfoPage;
    }

    @Override
    public Page<TEventOptimizeInfoVO> selectPageWithCrossIdAndGreenId(int currentPage, int pageSize, String startTime, String endTime, String crossId, Integer greenId) {
        Page<TEventOptimizeInfo> page = new Page<>(currentPage, pageSize);
        Page<TEventOptimizeInfo> tEventOptimizeInfoPage = baseMapper.selectPageWithCrossIdAndGreenId(page, startTime, endTime, crossId, greenId);
        List<TEventOptimizeInfoVO> collect = tEventOptimizeInfoPage.getRecords().stream().map(x -> {
            TEventOptimizeInfoVO tEventOptimizeInfoVO = new TEventOptimizeInfoVO();
            BeanUtils.copyProperties(x, tEventOptimizeInfoVO);
            tEventOptimizeInfoVO.setEventLabal(getEventLabel(x.getEventType()));
            return tEventOptimizeInfoVO;
        }).collect(Collectors.toList());

        Page<TEventOptimizeInfoVO> pageVO = new Page<>(currentPage, pageSize);
        pageVO.setTotal(page.getTotal());
        pageVO.setRecords(collect);
        return pageVO;
    }

    @Override
    public JSONObject getCrossOptimizeDistribute(Integer greenId, String crossId, String startTime, String endTime, Integer groupType) {
        List<Map<String, String>> crossOptimizeDistribute = tEventOptimizeInfoMapper.getCrossOptimizeDistribute(groupType, startTime, endTime, crossId,greenId);
        List<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxisAll(String.valueOf(groupType), startTime, endTime);// 按事件类型分组
        Map<String, EventOptimizeCountPo> eventTypeMap = new HashMap<>();
        for (Map<String, String> data : crossOptimizeDistribute) {
            String eventType = data.get("event_type");
            EventOptimizeCountPo optimizeCountPo = eventTypeMap.computeIfAbsent(eventType, k -> {
                EventOptimizeCountPo item = new EventOptimizeCountPo();
                item.setEventLabel(getEventLabel(eventType)); // 设置事件名称
                item.setEventType(eventType);
                item.setNoOptimizeNumberList(new ArrayList<>(Collections.nCopies(sortedSet.size(), 0)));
                item.setTotalNumberList(new ArrayList<>(Collections.nCopies(sortedSet.size(), 0)));
                return item;
            });

            // 更新对应时间的统计数据
            int timeIndex = sortedSet.indexOf(data.get("time_label"));
            String totalCount = String.valueOf(data.get("total_count")) ;
            String unoptimizedCount = String.valueOf(data.get("unoptimized_count")) ;
            optimizeCountPo.getTotalNumberList().set(timeIndex, Integer.valueOf(totalCount));
            optimizeCountPo.getNoOptimizeNumberList().set(timeIndex, Integer.valueOf(unoptimizedCount));
        }
        ArrayList<String> timeLabelList = new ArrayList<>(Collections.nCopies(sortedSet.size(), ""));
        sortedSet.forEach(x->{
            String timeFormat = getTimeFormat(groupType,x);
            int index = sortedSet.indexOf(x);
            timeLabelList.set(index,timeFormat);
        });
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", timeLabelList);
        jsonObject.put("timeListAll", sortedSet);
        jsonObject.put("dataList", eventTypeMap.values());


        return jsonObject;
    }

    private String getTimeFormat(Integer groupType, String time) {
        DateTime dateTime = DateTime.parse(time, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER));
        String format = "";
        if(!ObjectUtils.isEmpty(groupType) && groupType == 4 ){
            format = dateTime.toString("yyyy-MM-dd");
        }else {
            format = dateTime.toString("HH:mm");
        }


        return format;
    }

    // 获取事件名称（可以根据实际需求扩展）
    private String getEventLabel(String eventType) {
        String eventLabel = tEventOptimizeInfoMapper.getEventLabel(eventType);
        if(ObjectUtils.isEmpty(eventLabel)){
            return "未知事件";
        }else {
            return eventLabel;
        }
    }

    // 新增干线接口用于带有条件的分页查询
    @Override
    public Page<TEventOptimizeInfo> pageWithLineConditions(int currentPage, int pageSize, String startTime, String endTime, Integer greenId, String eventType, Integer optStatus, Integer duration,Integer startOptDuration,Integer endOptDuration) {
        Page<TEventOptimizeInfo> page = new Page<>(currentPage, pageSize);
        return tEventOptimizeInfoMapper.pageWithLineConditions(page,startTime,endTime,greenId, eventType, optStatus ,duration,startOptDuration,endOptDuration);
    }

}