package net.wanji.opt.servicev2.judgeanalysis.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemAndStrategyDayMapper;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemAndStrategyDay;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemAndStrategyDayService;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * <p>
 * 事件问题于策略关系实现类
 * </p>
 *
 * @author hwm
 * @since 2025-03-20
 */
@Service
@Slf4j
public class AnalysisProblemAndStrategyDayImpl extends ServiceImpl<AnalysisProblemAndStrategyDayMapper, AnalysisProblemAndStrategyDay> implements AnalysisProblemAndStrategyDayService {
    @Resource
    private AnalysisProblemAndStrategyDayMapper analysisProblemAndStrategyDayMapper;


    public void selectCountByType() {
        StopWatch stopWatch = new StopWatch("事件优化时长计算监控");
        stopWatch.start();
        //查询路口事件数据
        List<AnalysisProblemAndStrategyDay> crossEvenList = analysisProblemAndStrategyDayMapper.selectCrossEvent();
        if (crossEvenList.size() > 0) {
            //循环查找路口事件数据是否有策略下发
            for (AnalysisProblemAndStrategyDay vo : crossEvenList) {
                AnalysisProblemAndStrategyDay temp = analysisProblemAndStrategyDayMapper.getCrossStrategy(vo.getCrossId(), vo.getHappenStartTime(),vo.getEventSerialNumber());
                if (temp == null) {
                    vo.setOptStatus(0);
                } else {
                    vo.setOptStatus(1);
                    vo.setOptStartTime(temp.getOptStartTime());
                    vo.setOptEndTime(temp.getOptEndTime());
                    vo.setOptDuration(temp.getOptDuration());
                }

                LocalDate previousDay = LocalDate.now().minusDays(1);
                String formattedDate = previousDay.format(DateTimeFormatter.ofPattern("yyyyMMdd"));
                vo.setDt(formattedDate);
                vo.setInsertTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));

                //写入数据--Error
                analysisProblemAndStrategyDayMapper.insertProblemAndStrategy(vo);
            }
        }

        //查询干线事件数据
        List<AnalysisProblemAndStrategyDay> greenEvenList = analysisProblemAndStrategyDayMapper.selectGreenEvent();
        if (greenEvenList.size() > 0) {
            for (AnalysisProblemAndStrategyDay vo : greenEvenList) {
                AnalysisProblemAndStrategyDay temp = analysisProblemAndStrategyDayMapper.getGreenStrategy(vo.getGreenId(), vo.getHappenStartTime());
                if (temp == null) {
                    vo.setOptStatus(0);
                } else {
                    vo.setOptStatus(1);
                    vo.setOptStartTime(temp.getOptStartTime());
                    vo.setOptEndTime(temp.getOptEndTime());
                    vo.setOptDuration(temp.getOptDuration());
                }
                LocalDate previousDay = LocalDate.now().minusDays(1);
                String formattedDate = previousDay.format(DateTimeFormatter.ofPattern("yyyyMMdd"));
                vo.setDt(formattedDate);
                vo.setInsertTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));

                //写入数据
                analysisProblemAndStrategyDayMapper.insertProblemAndStrategy(vo);
            }
        }
        stopWatch.stop();
        log.info("事件优化记录优化耗时：{}ms",stopWatch.getTotalTimeMillis());

    }

}
