package net.wanji.opt.servicev2.report.impl;


import net.wanji.common.enums.WeekEnum;
import net.wanji.common.framework.dubbointerface.impl.BaseDubboInterfaceImpl;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.mapper.BaseInterfaceMapper;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.opt.common.ArithOfBigDecmial;
import net.wanji.opt.common.enums.PeakNameEnum;
import net.wanji.opt.constant.PeakEnum;
import net.wanji.opt.constant.WeekDayEnum;
import net.wanji.opt.entity.report.AnalysisGreenWavePeakDetail;
import net.wanji.opt.servicev2.report.AnalysisGreenWavePeakDetailService;
import net.wanji.opt.dao.mapper.report.AnalysisGreenWavePeakDetailMapper;

import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.vo2.report.GreenCongestTimeSpanVO;
import net.wanji.opt.vo2.report.GreenReportRunStateResponseVO;
import net.wanji.opt.vo2.report.GreenReportRunStatedDataVO;
import org.joda.time.DateTime;
import org.joda.time.Minutes;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * <p>
 *
 * </p>
 *
 * @Author fengyi
 * @Date 2025-04-07
 */
@Slf4j
@Component
@Service
public class AnalysisGreenWavePeakDetailServiceImpl extends BaseDubboInterfaceImpl<AnalysisGreenWavePeakDetail> implements AnalysisGreenWavePeakDetailService {

    @Resource
    private AnalysisGreenWavePeakDetailMapper analysisGreenWavePeakDetailMapper;
    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;

    @Override
    public BaseInterfaceMapper<AnalysisGreenWavePeakDetail> getBaseInterfaceMapper() {
        return this.analysisGreenWavePeakDetailMapper;
    }

    @Override
    public List<GreenReportRunStateResponseVO> selectWeekRunStateSituation(String ids, Integer year, Integer week) {
        List<GreenReportRunStateResponseVO> retList = new ArrayList<>();

        List<Integer> greenList = Stream.of(ids.split(",")).map(o -> Integer.valueOf(o)).collect(Collectors.toList());

        Integer yearWeek = Integer.valueOf(year.toString() + week.toString());
        Integer lastWeek = yearWeek;//Integer.valueOf(year.toString() + (week - 1));

        Map<String, Object> params = new HashMap<>();
        params.put("greenIdList", greenList);
        params.put("yearWeek", yearWeek);
        //本周数据
        List<AnalysisGreenWavePeakDetail> thisWeekList = analysisGreenWavePeakDetailMapper.findByMap(params);
        Map<Integer, List<AnalysisGreenWavePeakDetail>> thisWeekDataById = thisWeekList.stream().collect(Collectors.groupingBy(o -> o.getGreenId()));

        params.put("yearWeek", lastWeek);
        //上周数据
        List<AnalysisGreenWavePeakDetail> lastWeekList = analysisGreenWavePeakDetailMapper.findByMap(params);
        Map<Integer, List<AnalysisGreenWavePeakDetail>> lastWeekDataById = lastWeekList.stream().collect(Collectors.groupingBy(o -> o.getGreenId()));

        List<GreenwaveInfoPO> greenInfoList = greenwaveInfoMapper.selectAll();
        Map<Integer, String> greenNameMap = greenInfoList.stream().collect(Collectors.toMap(GreenwaveInfoPO::getId, GreenwaveInfoPO::getName));
        //遍历本周各条干线
        thisWeekDataById.forEach((k, v) -> {
            //上周数据
            List<AnalysisGreenWavePeakDetail> lastWeekDataList = lastWeekDataById.get(k);

            //总体运行情况
            GreenReportRunStateResponseVO runOverviewData = this.getRunOverview(v, lastWeekDataList);
            runOverviewData.setGreenId(k+"");
            runOverviewData.setGreenName(greenNameMap.get(k));

            //按星期、峰期类型排序
            List<AnalysisGreenWavePeakDetail> sortByWeekPeakList = v.stream().sorted(Comparator.comparing(o -> o.getWeekDay() + "_" + o.getPeakType())).collect(Collectors.toList());
            List<GreenReportRunStatedDataVO> tableDataList = new ArrayList<>();
            //输出每周几峰值阶段相关数据
            for (AnalysisGreenWavePeakDetail detail : sortByWeekPeakList) {

                GreenReportRunStatedDataVO weekData = new GreenReportRunStatedDataVO();

                weekData.setWeekName(WeekDayEnum.getNameByCode(detail.getWeekDay()).getName());
                weekData.setPeakName(PeakEnum.getNameByCode(Integer.valueOf(detail.getPeakType())).getName());
                weekData.setThisWeekAvgSpeed(detail.getSpeed());
                weekData.setThisWeekCongestIndex(detail.getTrafficIndex());
                weekData.setThisWeekTravelTime(detail.getTravelTime() == null ? 0D : ArithOfBigDecmial.div(detail.getTravelTime().doubleValue(), 60));
                weekData.setThisWeekDayPeakSpan(DateUtil.formatDate(detail.getPeakStartTime(), "HH:mm") + "~" + DateUtil.formatDate(detail.getPeakEndTime(), "HH:mm"));
                //本周峰期内拥堵时段数据获取
                List<GreenCongestTimeSpanVO> congestTimeSpan1 = analysisGreenWavePeakDetailMapper.findCongestTimeSpanInPeakTime(yearWeek, k, detail.getPeakStartTime(), detail.getPeakEndTime());
                GreenCongestTimeSpanVO vo1 = congestTimeSpan1.stream().min(Comparator.comparing(GreenCongestTimeSpanVO::getCongestStartTime)).orElse(new GreenCongestTimeSpanVO());
                GreenCongestTimeSpanVO vo2 = congestTimeSpan1.stream().max(Comparator.comparing(GreenCongestTimeSpanVO::getCongestEndTime)).orElse(new GreenCongestTimeSpanVO());
                if (vo1.getCongestStartTime() !=null && vo2.getCongestEndTime() !=null) {
                    weekData.setThisWeekCongestSpan(DateUtil.formatDate(vo1.getCongestStartTime(), "HH:mm") + "~" + DateUtil.formatDate(vo2.getCongestEndTime(), "HH:mm"));
                }
                //上周高峰指标数据
                Optional<AnalysisGreenWavePeakDetail> optional = lastWeekDataList.stream().filter(o -> Objects.equals(o.getWeekDay(), detail.getWeekDay()) && Objects.equals(o.getPeakType(), detail.getPeakType())).findFirst();
                if (optional.isPresent()) {
                    weekData.setLastWeekAvgSpeed(optional.get().getSpeed());
                    weekData.setLastWeekCongestIndex(optional.get().getTrafficIndex());
                    weekData.setLastWeekTravelTime(optional.get().getTravelTime() == null ? 0D : ArithOfBigDecmial.div(optional.get().getTravelTime().doubleValue(), 60));
                    //上周峰期内拥堵时段数据获取
                    List<GreenCongestTimeSpanVO> congestTimeSpan2 = analysisGreenWavePeakDetailMapper.findCongestTimeSpanInPeakTime(lastWeek, k, optional.get().getPeakStartTime(), optional.get().getPeakEndTime());
                    GreenCongestTimeSpanVO vo3 = congestTimeSpan2.stream().min(Comparator.comparing(GreenCongestTimeSpanVO::getCongestStartTime)).orElse(new GreenCongestTimeSpanVO());
                    GreenCongestTimeSpanVO vo4 = congestTimeSpan2.stream().max(Comparator.comparing(GreenCongestTimeSpanVO::getCongestEndTime)).orElse(new GreenCongestTimeSpanVO());
                    if (vo3.getCongestStartTime() !=null && vo4.getCongestEndTime() !=null) {
                        weekData.setLastWeekCongestSpan(DateUtil.formatDate(vo3.getCongestStartTime(),"HH:mm")+"~"+DateUtil.formatDate(vo4.getCongestEndTime(),"HH:mm"));
                    }
                    weekData.setLastWeekDayPeakSpan(DateUtil.formatDate(optional.get().getPeakStartTime(), "HH:mm") + "~" + DateUtil.formatDate(optional.get().getPeakEndTime(), "HH:mm"));
                }

                tableDataList.add(weekData);
            }

            runOverviewData.setDataList(tableDataList);
            retList.add(runOverviewData);
        });
        return retList;
    }

    /**
     * 周运行概况同比
     *
     * @param thisWeekDataList 本周数据
     * @param lastWeekDataList 上周数据
     */
    public GreenReportRunStateResponseVO getRunOverview(List<AnalysisGreenWavePeakDetail> thisWeekDataList, List<AnalysisGreenWavePeakDetail> lastWeekDataList) {

        GreenReportRunStateResponseVO amPeakData = this.analysisWeekAmPeakSituation(thisWeekDataList, lastWeekDataList, 1);
        GreenReportRunStateResponseVO pmPeakData = this.analysisWeekAmPeakSituation(thisWeekDataList, lastWeekDataList, 2);

        GreenReportRunStateResponseVO ret = new GreenReportRunStateResponseVO();
        BeanUtils.copyProperties(amPeakData, ret);
        BeanUtils.copyProperties(pmPeakData, ret,new String[]{"thisWeekPeakEarliestStartTime","thisWeekPeakLatestEndTime","weekAmPeakSpan","amStartSameRatioSituation","amEndSameRatioSituation","amDuration","amDurationSameRatioSituation","amMaxTravelTime","amMaxTravelTimeSameRatioSituation","amAvgContestIndex","amAvgContestIndexSameRatioSituation"});

        return ret;
    }

    /**
     * 分析周高峰时段情况
     *
     * @param thisWeekList     本周数据
     * @param lastWeekDataList 上周数据
     * @param peakType         1早高峰 2晚高峰 3平峰
     * @return
     */
    private GreenReportRunStateResponseVO analysisWeekAmPeakSituation(List<AnalysisGreenWavePeakDetail> thisWeekList, List<AnalysisGreenWavePeakDetail> lastWeekDataList, Integer peakType) {
        GreenReportRunStateResponseVO vo = new GreenReportRunStateResponseVO();

        //本周早高峰最早开始时间
        AnalysisGreenWavePeakDetail earliestStartTimePo = this.getPeakTimeBoundary(thisWeekList, 1, peakType);
        //本周早高峰最晚结束时间
        AnalysisGreenWavePeakDetail lastEndTimePo = this.getPeakTimeBoundary(thisWeekList, 2, peakType);
        //上周早高峰最早开始时间
        AnalysisGreenWavePeakDetail lastWeekEarliestStartTimePo = this.getPeakTimeBoundary(lastWeekDataList, 1, peakType);
        //上周早高峰最晚结束时间
        AnalysisGreenWavePeakDetail lastWeekLastEndTimePo = this.getPeakTimeBoundary(lastWeekDataList, 2, peakType);
        //本周与上周高峰最早开始时间差距
        int minutesStart = Minutes.minutesBetween(new DateTime(earliestStartTimePo.getPeakStartTime().getTime()), new DateTime(lastWeekEarliestStartTimePo.getPeakStartTime().getTime())).getMinutes();
        //本周与上周高峰最晚结束时间差距
        int minutesEnd = Minutes.minutesBetween(new DateTime(earliestStartTimePo.getPeakEndTime().getTime()), new DateTime(lastWeekEarliestStartTimePo.getPeakEndTime().getTime())).getMinutes();

        //====================================================================================================================================================//
        //早高峰集中时段
        String amPeakScope = DateUtil.formatDate(earliestStartTimePo.getPeakStartTime(), "HH:mm") + "~" + DateUtil.formatDate(lastEndTimePo.getPeakEndTime(), "HH:mm");
        //====================================================================================================================================================//
        //开始时间比上周
        String amPeakStartSituation = "";
        if (minutesStart > 0) {
            amPeakStartSituation = "提前" + minutesStart + "分钟";
        } else {
            amPeakStartSituation = "延后" + minutesStart + "分钟";
        }
        //====================================================================================================================================================//
        //结束时间比上周
        String amPeakEndSituation = "";
        if (minutesEnd > 0) {
            amPeakEndSituation = "提前" + minutesEnd + "分钟";
        } else {
            amPeakEndSituation = "延后" + minutesEnd + "分钟";
        }
        //====================================================================================================================================================//
        //持续时长
        int amDuration1 = Minutes.minutesBetween(new DateTime(earliestStartTimePo.getPeakStartTime().getTime()), new DateTime(lastEndTimePo.getPeakEndTime().getTime())).getMinutes();
        //持续时长
        int amDuration2 = Minutes.minutesBetween(new DateTime(lastWeekEarliestStartTimePo.getPeakStartTime().getTime()), new DateTime(lastWeekLastEndTimePo.getPeakEndTime().getTime())).getMinutes();
        //本周高峰持续时长
        String amPeakDuration = amDuration1 + "分钟";
        String amDurationSameRatioSituation = "";
        if (amDuration1 > amDuration2) {
            amDurationSameRatioSituation = "增加" + (amDuration1 - amDuration2) + "分钟";
        } else {
            amDurationSameRatioSituation = "减少" + (amDuration2 - amDuration1) + "分钟";
        }
        //====================================================================================================================================================//
        //本周最大行程时间
        Integer thisMaxTravelTime = thisWeekList.stream().map(AnalysisGreenWavePeakDetail::getTravelTime).max(Comparator.comparing(Integer::intValue)).orElse(0);
        //上周最大行程时间
        Integer lastMaxTravelTime = lastWeekDataList.stream().map(AnalysisGreenWavePeakDetail::getTravelTime).max(Comparator.comparing(Integer::intValue)).orElse(0);
        //最大行程时间状况
        String maxTravelTimeSituation = "";
        if (thisMaxTravelTime > lastMaxTravelTime) {
            maxTravelTimeSituation = "增加" + (Math.abs(thisMaxTravelTime - lastMaxTravelTime)) + "分钟";
        } else {
            maxTravelTimeSituation = "减少" + (Math.abs(thisMaxTravelTime - lastMaxTravelTime)) + "分钟";
        }
        //====================================================================================================================================================//
        //本周平均拥堵指数
        Double thisAvgCongestIndex = thisWeekList.stream().map(AnalysisGreenWavePeakDetail::getTrafficIndex).max(Comparator.comparing(Double::doubleValue)).orElse(0D);
        //上周平均拥堵指数
        Double lastAvgCongestIndex = lastWeekDataList.stream().map(AnalysisGreenWavePeakDetail::getTrafficIndex).max(Comparator.comparing(Double::doubleValue)).orElse(0D);
        //平均拥堵指数状况
        String amAvgCongestIndex = "";
        if (thisAvgCongestIndex > lastAvgCongestIndex) {
            amAvgCongestIndex = "增加" + (Math.abs(thisAvgCongestIndex - lastAvgCongestIndex)) + "";
        } else {
            amAvgCongestIndex = "减少" + (Math.abs(thisAvgCongestIndex - lastAvgCongestIndex)) + "";
        }
        //====================================================================================================================================================//
        if (Objects.equals(peakType, 1)) {
            vo.setWeekAmPeakSpan(amPeakScope);
            vo.setAmStartSameRatioSituation(amPeakStartSituation);
            vo.setAmEndSameRatioSituation(amPeakEndSituation);
            vo.setAmDuration(amPeakDuration);
            vo.setAmDurationSameRatioSituation(amDurationSameRatioSituation);
            vo.setAmMaxTravelTime(thisMaxTravelTime + "分钟");
            vo.setAmMaxTravelTimeSameRatioSituation(maxTravelTimeSituation);
            vo.setAmAvgContestIndex(thisAvgCongestIndex);
            vo.setAmAvgContestIndexSameRatioSituation(amAvgCongestIndex);
            vo.setThisWeekPeakEarliestStartTime(earliestStartTimePo.getPeakStartTime());
            vo.setThisWeekPeakLatestEndTime(lastEndTimePo.getPeakEndTime());
        } else if (Objects.equals(peakType, 2)) {
            vo.setWeekPmPeakSpan(amPeakScope);
            vo.setPmStartSameRatioSituation(amPeakStartSituation);
            vo.setPmEndSameRatioSituation(amPeakEndSituation);
            vo.setPmDuration(amPeakDuration);
            vo.setPmDurationSameRatioSituation(amDurationSameRatioSituation);
            vo.setPmMaxTravelTime(thisMaxTravelTime + "分钟");
            vo.setPmMaxTravelTimeSameRatioSituation(maxTravelTimeSituation);
            vo.setPmAvgContestIndex(thisAvgCongestIndex);
            vo.setPmAvgContestIndexSameRatioSituation(amAvgCongestIndex);
            vo.setLastWeekPeakEarliestStartTime(earliestStartTimePo.getPeakStartTime());
            vo.setLastWeekPeakLatestEndTime(lastEndTimePo.getPeakEndTime());
        }

        return vo;
    }


    /**
     * 获取峰值时间边界
     *
     * @param list     包含波峰详情的列表
     * @param type     类型 1：高峰开始时间 2高峰截止时间
     * @param peakType 波峰类型，1高峰 2晚峰 3平峰
     * @return 符合条件的波峰详情对象；如果没有找到符合条件的对象，则返回null
     */
    private AnalysisGreenWavePeakDetail getPeakTimeBoundary(List<AnalysisGreenWavePeakDetail> list, Integer type, Integer peakType) {
        AnalysisGreenWavePeakDetail ret = null;
        //修改为取交通指数最大的一条记录
        Optional<AnalysisGreenWavePeakDetail> optional =  list.stream().filter(o -> Objects.equals(peakType.toString(), o.getPeakType())).min(Comparator.comparing(AnalysisGreenWavePeakDetail::getTrafficIndex).reversed());
        if (optional.isPresent()){
            ret = optional.get();
        }
        //多个时段，取第一个时段的第一个开始时段，最后一个时段的截止时段
//        if (Objects.equals(1, type)) {
//            Optional<AnalysisGreenWavePeakDetail> optional =  list.stream().filter(o -> Objects.equals(peakType.toString(), o.getPeakType())).min(Comparator.comparing(o -> o.getPeakStartTime()));
//            if (optional.isPresent()){
//                ret = optional.get();
//            }
//        } else {
//            Optional<AnalysisGreenWavePeakDetail> optional  = list.stream().filter(o -> Objects.equals(peakType.toString(), o.getPeakType())).max(Comparator.comparing(o -> o.getPeakEndTime())) ;
//            if (optional.isPresent()){
//                ret = optional.get();
//            }
//        }
        return ret;
    }

}
