package net.wanji.opt.servicev2.strategy.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.strategy.StrategyPriorityMapper;
import net.wanji.opt.entity.strategy.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.wanji.opt.entity.strategy.dto.StrategyPriorityGroup;
import net.wanji.opt.servicev2.strategy.StrategyPriorityService;
import net.wanji.opt.synthesis.pojo.StrategyFactoryEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
* <p>
* 策略优先级日计划详情表 服务实现类
* </p>
*
* @author fengyi
* @since 2025-03-28
*/
@Service
@Slf4j
 public class StrategyPriorityServiceImpl extends ServiceImpl<StrategyPriorityMapper, StrategyPriorityDailyInfo> implements StrategyPriorityService {

	 @Resource
	 private StrategyPriorityMapper strategyPriorityMapper;

	
	 /**
	 * 策略优先级日计划详情表详情
	 * @param id
	 * @return
	 */
	 @Override
	 public StrategyPriorityDailyInfo info(Long id) {
	
	 	return getById(id);
	 }
	
	 /**
	 * 策略优先级日计划详情表新增
	 * @param param 根据需要进行传值
	 * @return
	 */
	 @Override
	 public void add(StrategyPriorityDailyInfo param) {
	
	 	save(param);
	 }
	
	 /**
	 * 策略优先级日计划详情表修改
	 * @param param 根据需要进行传值
	 * @return
	 */
	 @Override
	 public void modify(StrategyPriorityDailyInfo param) {
	
	 	updateById(param);
	 }
	
	 /**
	 * 策略优先级日计划详情表删除(单个条目)
	 * @param id
	 * @return
	 */
	 @Override
	 public void remove(Long id) {
	 	removeById(id);
	 }
	
	 /**
	 * 策略优先级日计划详情表删除(多个条目)
	 * @param ids
	 * @return
	 */
	 @Override
	 public void removes(List<Long> ids) {
	
	 	removeByIds(ids);
	 }


	/**
	 * 策略路口列表
	 * @param
	 * @return
	 */
	@Override
	public  List<StrategyPriorityDailyInfo> pageStrategyCrossList() {
		 List<StrategyPriorityDailyInfo> allList= strategyPriorityMapper.pageStrategyCrossList();

		 return allList ;
	 }

	/**
	 * 策略名称
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getStrategyList() {
		List<StrategyFactoryEntity> strategyList= strategyPriorityMapper.getStrategyList();
		return strategyList;
	}
	/**
	 * 场景
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getSceneList() {
		List<StrategyFactoryEntity> strategyList= strategyPriorityMapper.getSceneList();
		return strategyList;
	}
	/**
	 * 厂商
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getCompanyList() {
		List<StrategyFactoryEntity> strategyList=strategyPriorityMapper.getCompanyList();
		return strategyList;
	}

	@Override
	@Transactional
	public void savePriority(List<StrategyPriorityGroup> dataList) {
		try {
			List<StrategyPriorityConfig> saveList = new ArrayList<>();
			for (StrategyPriorityGroup group : dataList) {
				String label = group.getLabel();
				Integer labelCode = group.getLabelCode();
				//路口id
				List<String> crossList = group.getCrossIds();
				//选中多个路口存多条数据
				for (String crossId : crossList) {
					//先判断表里有没有数据，有的话删除t_strategy_priority_config
					List<StrategyPriorityConfig> priorityConfig = strategyPriorityMapper.selectPriorityTable(crossId);
					if (priorityConfig.size() > 0) {
						strategyPriorityMapper.deletePriorityConfig(crossId);
					}

					if (CollectionUtils.isNotEmpty(group.getData())) {
						for (StrategyPriorityConfig item : group.getData()) {
							StrategyPriorityConfig strategyPriorityConfig = new StrategyPriorityConfig();
							strategyPriorityConfig.setCrossId(crossId);
							//分组id
							strategyPriorityConfig.setGroupId(labelCode);
							//分组名称
							strategyPriorityConfig.setGroupName(label);
							//策略编号
							strategyPriorityConfig.setStrategyNo(item.getStrategyNo());
							//优先级
							strategyPriorityConfig.setPriority(item.getPriority());

							saveList.add(strategyPriorityConfig);
						}
					}
				}
			}
			strategyPriorityMapper.savePriorityInsert(saveList);
		} catch (Exception e) {
			log.error("{} savePriority", this.getClass().getSimpleName(), e);
			throw  e;
		}
	}

	@Override
	@Transactional
	public List<StrategyFactoryEntity> getPriorityData(String crossId) {

		try {
			//先获取策略编号
			List<String> strategyNo = strategyPriorityMapper.getstrategyNo(crossId);
			if (CollectionUtils.isEmpty(strategyNo)){
				strategyNo.add("0");
			}
			List<StrategyFactoryEntity> priorityList = strategyPriorityMapper.getPriorityConfigData(strategyNo,crossId);
			return  priorityList;
		}catch (Exception e){
			log.error("{} getPriorityData", this.getClass().getSimpleName(), e);
			throw  e;
		}

	}

	@Override
	@Transactional
	public void savePlanConfig(List<StrategyPriorityGroup> dailyPlanDetails) throws JsonProcessingException {

		try {
			List<StrategyPriorityDailyInfo> savePlanList = new ArrayList<>();
			ObjectMapper objectMapper = new ObjectMapper();
			for (StrategyPriorityGroup group : dailyPlanDetails) {
				//路口ID
				List<String> crossList = group.getCrossIds();
				//选中多个路口存多条数据
				for (String crossId : crossList) {
					StrategyPriorityDailyInfo strategyPriorityDailyInfo = new StrategyPriorityDailyInfo();
					strategyPriorityDailyInfo.setCrossId(crossId);
					//日计划编号
					Integer dailyPlanId = group.getDailyPlanId();
					strategyPriorityDailyInfo.setDailyPlanId(dailyPlanId);
					//执行时间
					String weekExecute = group.getWeekExecute();
					strategyPriorityDailyInfo.setWeekExecute(weekExecute);
					//先判断表里有没有数据，有的话删除t_strategy_priority_daily_info
					List<StrategyPriorityDailyInfo> planConfig = strategyPriorityMapper.selectPlanTable(crossId);
					if (planConfig.size() > 0) {
						strategyPriorityMapper.deletePlanConfig(crossId);
					}

					//取出日计划详情
					List<StrategyPriorityPlanDetails> dailyPlanDetails1 = group.getDailyPlanDetails();
					String dailyPlanDetailsJson = JSON.toJSONString(dailyPlanDetails1);
					strategyPriorityDailyInfo.setDailyPlanDetails(dailyPlanDetailsJson);

					savePlanList.add(strategyPriorityDailyInfo);
				}
			}
			strategyPriorityMapper.savePlanConfig(savePlanList);
		}catch (Exception e){
			log.error("{} savePlanConfig", this.getClass().getSimpleName(), e);
			throw  e;
		}
	}

	@Override
	public List<StrategyPriorityDailyInfo> getPlanConfigData(String crossId) {

		List<StrategyPriorityDailyInfo> planList = strategyPriorityMapper.getPlanConfigData(crossId);
		return planList;
	}
/*
*  策略参数配置
* */
	@Override
	@Transactional
	public void saveParamterConfig(StrategyPriorityGroup group) throws JsonProcessingException {

		try {
			List<StrategyParameterConfig> savePlanList = new ArrayList<>();
			StrategyParameterConfig strategyParameterConfig = new StrategyParameterConfig();
				//策略编号
			String strategyNo = group.getStrategyNo();
			strategyParameterConfig.setStrategyNo(strategyNo);
				//路口id
			String crossId = group.getCrossId();
			strategyParameterConfig.setCrossId(crossId);
				//先判断表里有没有数据，有的话删除t_strategy_priority_parameter
				List<StrategyParameterConfig> paramterConfig = strategyPriorityMapper.paramterConfigTable(crossId);
				if (paramterConfig.size() > 0) {
					strategyPriorityMapper.deleteParamterConfig(crossId);
				}
				//参数 场景配置
			List<StrategyPriorityParamter> parameterConfigList = group.getParameterConfigList();
			String jsonString = JSON.toJSONString(parameterConfigList);
			strategyParameterConfig.setParamDetails(jsonString);
			//策略调度参数配置
			StrategySchedulingParam schedulingParamters = group.getSchedulingParamters();
			ObjectMapper objectMapper = new ObjectMapper();
			String schedulingParamter = objectMapper.writeValueAsString(schedulingParamters);
			strategyParameterConfig.setSchedulingParam(schedulingParamter);
			//添加到最终数据
			savePlanList.add(strategyParameterConfig);
			strategyPriorityMapper.saveParamConfig(savePlanList);
		}catch (Exception e){
			log.error("{} saveParamterConfig", this.getClass().getSimpleName(), e);
			throw  e;
		}
	}
	/*
	 *  策略参数配置数据回显
	 * */
	@Override
	public List<StrategyParameterConfig> getParamConfigData(String crossId) {
         try {
			List<StrategyParameterConfig> paraterConfigList = strategyPriorityMapper.getParamConfigData(crossId);
			return paraterConfigList;
		}catch (Exception e){
			log.error("{} getParamConfigData", this.getClass().getSimpleName(), e);
			throw  e;
		}

	}
}
