package net.wanji.opt.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenCrossDirDTO;
import net.wanji.databus.dao.entity.GreenwaveRealtimePO;
import net.wanji.databus.dao.mapper.GreenwaveRealtimeMapper;
import net.wanji.opt.cache.GreenCrossDirInfoCache;
import net.wanji.opt.common.CommonUtils;
import net.wanji.opt.common.RedisUtils;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.dto.GreenBeltChartDTO;
import net.wanji.opt.dto.RingPhaseInfoDTO;
import net.wanji.opt.dto.SchemeGreenDirGreenTimeDTO;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2025/04/01 11:18
 * @description 海信通过reids缓存方案相位信息，设置绿波时序图
 */
@Component
@Slf4j
@Profile("test")
public class HisenseGreenChartTask {

    @Resource
    private RedisUtils redisUtils;
    @Resource
    private GreenwaveRealtimeMapper greenwaveRealtimeMapper;


    //@Scheduled(fixedRate = 10 * 60 * 1000, initialDelay = 1 * 60 * 1000)
    @Scheduled(fixedRate = 1 * 60 * 1000, initialDelay = 30 * 1000)
    public void getGreenChart() {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            Map<String, GreenwaveRealtimePO> realTimeMap = getGreenRealTimeMap();
            // 绿波路口数据
            List<GreenCrossDirDTO> greenCrossDirCache = GreenCrossDirInfoCache.greenCrossDirCache;
            List<SchemeGreenDirGreenTimeDTO> schemeGreenDirGreenTimeDTOS = getSchemeGreenDirGreenTimeDTOS(greenCrossDirCache);
            if (!CollectionUtils.isEmpty(greenCrossDirCache) && !CollectionUtils.isEmpty(schemeGreenDirGreenTimeDTOS)) {
                // 将海信方案环相位转成map形式
                HashMap<String, SchemeGreenDirGreenTimeDTO> greenDirGreenTimeMap = new HashMap<>();
                for (SchemeGreenDirGreenTimeDTO dto : schemeGreenDirGreenTimeDTOS) {
                    String crossId = dto.getCrossId();
                    Integer dir = dto.getDir();
                    greenDirGreenTimeMap.put(CommonUtils.getStrJoin(crossId, String.valueOf(dir)), dto);
                }

                // 1.根据绿波带长度和速度计算行程时间
                // 2.根据相位差，行程时间，海信环图计算绿灯红灯点位 正向 crossRedTimesMap 反向backCrossRedTimesMap
                Map<Integer, List<GreenCrossDirDTO>> greenMap = greenCrossDirCache.stream().collect(Collectors.groupingBy(GreenCrossDirDTO::getGreenId));
                List<GreenBeltChartDTO> greenBeltChartDTOS = new ArrayList<>();
                for (Map.Entry<Integer, List<GreenCrossDirDTO>> entry : greenMap.entrySet()) {
                    GreenBeltChartDTO greenBeltChartDTO = new GreenBeltChartDTO();
                    Integer greenId = entry.getKey();
                    List<GreenCrossDirDTO> value = entry.getValue();
                    // 1.获取行程时间
                    GreenCrossDirDTO indexDTO = value.get(0);
                    Integer sort = indexDTO.getSort();
                    Integer indexDir = indexDTO.getInDir();
                    Integer totalDistance = indexDTO.getTotalDistance();
                    Double travelTime = getTravelTime(realTimeMap, greenId, indexDir, totalDistance);
                    List<Double> offsets = new ArrayList<>();
                    List<Double> distances = new ArrayList<>();
                    // 正向红灯时间
                    Map<String, List<List<Double>>> crossRedTimesMap = new LinkedHashMap<>();
                    Map<String, List<List<Double>>> crossGreenTimesMap = new LinkedHashMap<>();
                    getCrossRedTimeMap(mapper, greenDirGreenTimeMap, value, indexDir, travelTime, offsets, distances, crossRedTimesMap, crossGreenTimesMap);

                    // 反向红灯时间
                    Integer indexOutDir = indexDTO.getOutDir();
                    List<GreenCrossDirDTO> backValue = value.stream().sorted(Comparator.comparingInt(GreenCrossDirDTO::getSort).reversed()).collect(Collectors.toList());
                    List<Double> backOffsets = new ArrayList<>();
                    List<Double> backDistances = new ArrayList<>();
                    Double backTravelTime = getTravelTime(realTimeMap, greenId, indexOutDir, totalDistance);
                    Map<String, List<List<Double>>> backCrossRedTimesMap = new LinkedHashMap<>();
                    Map<String, List<List<Double>>> backCrossGreenTimesMap = new LinkedHashMap<>();
                    getCrossRedTimeMap(mapper, greenDirGreenTimeMap, backValue, indexOutDir, backTravelTime, backOffsets, backDistances, crossGreenTimesMap, backCrossGreenTimesMap);
                    greenBeltChartDTO.setGreenId(greenId);
                    greenBeltChartDTO.setDir(indexDir);
                    greenBeltChartDTO.setBackDir(indexOutDir);
                    greenBeltChartDTO.setSpeed(totalDistance / travelTime);
                    greenBeltChartDTO.setBackSpeed(totalDistance / backTravelTime);
                    greenBeltChartDTO.setOffsets(offsets);
                    greenBeltChartDTO.setBackOffsets(backOffsets);
                    greenBeltChartDTO.setDistances(distances);
                    greenBeltChartDTO.setBackDistances(backDistances);
                    greenBeltChartDTO.setCrossRedTimesMap(crossRedTimesMap);
                    greenBeltChartDTO.setBackCrossRedTimesMap(backCrossRedTimesMap);
                    greenBeltChartDTO.setCrossGreenTimesMap(crossGreenTimesMap);
                    greenBeltChartDTO.setBackCrossGreenTimesMap(backCrossGreenTimesMap);
                    greenBeltChartDTOS.add(greenBeltChartDTO);
                }

                // 3.计算每两个路口绿灯开始、结束时间
                for (Map.Entry<Integer, List<GreenCrossDirDTO>> entry : greenMap.entrySet()) {
                    Integer greenId = entry.getKey();
                    List<GreenCrossDirDTO> value = entry.getValue();
                    if (!CollectionUtils.isEmpty(value)) {
                        for (int i = 0; i < value.size(); i++) {
                            GreenCrossDirDTO firstDto = value.get(i);
                            String firstCrossId = firstDto.getCrossId();
                            Integer inDir = firstDto.getInDir();
                            if (i < value.size()) {
                                GreenCrossDirDTO secondDto = value.get(i + 1);
                                String secondCrossId = secondDto.getCrossId();
                                for (GreenBeltChartDTO greenBeltChartDTO : greenBeltChartDTOS) {
                                    Integer beltGreenId = greenBeltChartDTO.getGreenId();
                                    Integer beltDir = greenBeltChartDTO.getDir();
                                    if (Objects.equals(greenId, beltGreenId) && Objects.equals(inDir, beltDir)) {
                                        Double speed = greenBeltChartDTO.getSpeed();
                                        List<Double> distances = greenBeltChartDTO.getDistances();
                                        Double distance = distances.get(i);
                                        Double travelTime = distance / speed;
                                        Map<String, List<List<Double>>> crossGreenTimesMap = greenBeltChartDTO.getCrossGreenTimesMap();
                                        List<List<Double>> firstList = crossGreenTimesMap.get(firstCrossId);
                                        List<List<Double>> secondList = crossGreenTimesMap.get(secondCrossId);
                                        // 第一个路口循环绿灯开始时间
                                        for (int i1 = 0; i1 < firstList.size(); i1++) {
                                            List<Double> greenStartEnd = firstList.get(i1);
                                            for (int i2 = 0; i2 < greenStartEnd.size(); i2++) {
                                                Double greenStart = greenStartEnd.get(0);
                                                Double greenEnd = greenStartEnd.get(1);
                                                for (int j = greenStart.intValue(); j <= greenEnd.intValue(); j++) {
                                                    int firstStartPoint = j + travelTime.intValue();
                                                }

                                            }
                                        }

                                    }
                                }
                            }

                        }
                    }
                }

                log.error("绿波实体参数：{}", mapper.writeValueAsString(greenBeltChartDTOS));


            }
            log.error("=================");
        } catch (Exception e) {
            log.error("通过海信方案处理绿波波定时任务失败：{}", e);
            //throw new RuntimeException(e);
        }
    }

    @NotNull
    private static Double getTravelTime(Map<String, GreenwaveRealtimePO> realTimeMap, Integer greenId, Integer indexDir,
                                        Integer totalDistance) {
        String greenKey = CommonUtils.getStrJoin(String.valueOf(greenId), String.valueOf(indexDir));
        GreenwaveRealtimePO greenwaveRealtimePO = realTimeMap.get(greenKey);
        // km/h速度换成m/s
        Double speed = Objects.isNull(greenwaveRealtimePO.getSpeed()) ? 0 : greenwaveRealtimePO.getSpeed() / 3.6;
        Double travelTime = totalDistance / speed;
        return travelTime;
    }

    private static Map<String, List<List<Double>>> getCrossRedTimeMap(ObjectMapper mapper, HashMap<String, SchemeGreenDirGreenTimeDTO> greenDirGreenTimeMap,
                                                                      List<GreenCrossDirDTO> value, Integer indexDir, Double travelTime,
                                                                      List<Double> offsets, List<Double> distances, Map<String, List<List<Double>>> crossRedTimesMap,
                                                                      Map<String, List<List<Double>>> crossGreenTimesMap) throws Exception {
        for (GreenCrossDirDTO greenCrossDirDTO : value) {
            List<List<Double>> crossRedTimes = new ArrayList<>();
            List<List<Double>> crossGreenTimes = new ArrayList<>();
            String crossId = greenCrossDirDTO.getCrossId();
            Integer distance = greenCrossDirDTO.getDistance();
            distances.add(distance * 1.0);
            //String crossId = "13MQJ0B5SI0";
            String crossKey = CommonUtils.getStrJoin(String.valueOf(crossId), String.valueOf(indexDir));
            // 获取海信环相位信息
            SchemeGreenDirGreenTimeDTO firstGreenTimeDTO = greenDirGreenTimeMap.get(crossKey);
            Integer cycleLen = firstGreenTimeDTO.getCycleLen();
            // * 1.0转Double
            Double offset = firstGreenTimeDTO.getOffset() * 1.0;
            offsets.add(offset);

            Integer greenStartTime = firstGreenTimeDTO.getStart();
            Integer greenEndTime = firstGreenTimeDTO.getEnd();
            // 计算出绿灯结束时间到周期结束时间:
            Integer redEndTime = cycleLen - greenEndTime;
            // 通过行程时间计算周期数量
            double cycleSum = Math.floor(travelTime / cycleLen);
            for (int i = 0; i < cycleSum; i++) {
                Double cycleIndex = i * Double.valueOf(cycleLen);
                // 多个周期后，需要在最后补充的时间
                Double endCycleExt = cycleLen - offset;
                // 红灯结束时间 > 相位差, 相位差都是红灯,endCycleExt包含绿灯
                if (redEndTime >= offset) {
                    // offset+cycleLen 中红灯时间
                    crossRedTimes.add(Arrays.asList(cycleIndex, cycleIndex + offset + greenStartTime));
                    crossRedTimes.add(Arrays.asList(cycleIndex + offset + greenEndTime, cycleIndex + offset + cycleLen + greenStartTime));
                    // 两个周期，第二个周期是补充相位差
                    crossRedTimes.add(Arrays.asList(cycleIndex + offset + cycleLen + greenEndTime, cycleIndex + offset + cycleLen + endCycleExt));
                    // 绿灯时间
                    crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime, cycleIndex + offset + greenEndTime));
                    crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime + cycleLen, cycleIndex + offset + greenEndTime + cycleLen));

                } else { // 红灯结束时间 < 相位差, 相位差包含绿灯,endCycleExt都是红灯
                    // 相位差中绿灯时间
                    Double offsetGreen = greenEndTime - endCycleExt;
                    crossRedTimes.add(Arrays.asList(cycleIndex + offsetGreen, cycleIndex + offset + greenStartTime));
                    crossRedTimes.add(Arrays.asList(cycleIndex + offset + greenEndTime, cycleIndex + offset + cycleLen + greenStartTime));
                    // 绿灯时间
                    crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime, cycleIndex + offset + greenEndTime));
                }
            }
            crossRedTimesMap.put(crossId, crossRedTimes);
            crossGreenTimesMap.put(crossId, crossGreenTimes);
            log.debug("当前路口编号：{}，当前协调方向：{}， 红灯点位信息：{}", crossId, indexDir, mapper.writeValueAsString(crossRedTimes));
        }
        return crossRedTimesMap;
    }

    private Map<String, GreenwaveRealtimePO> getGreenRealTimeMap() {
        Map<String, GreenwaveRealtimePO> realTimeMap = new HashMap<>();
        LambdaQueryWrapper<GreenwaveRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        // todo 测试绿波1
        queryWrapper.eq(GreenwaveRealtimePO::getGreenId, 1);
        List<GreenwaveRealtimePO> greenwaveRealtimePOList = greenwaveRealtimeMapper.selectList(queryWrapper);
        for (GreenwaveRealtimePO greenwaveRealtimePO : greenwaveRealtimePOList) {
            Integer greenId = greenwaveRealtimePO.getGreenId();
            String roadDirection = greenwaveRealtimePO.getRoadDirection();
            Integer inDir = GreenBeltDirEnum.getInDir(roadDirection);
            realTimeMap.put(CommonUtils.getStrJoin(String.valueOf(greenId), String.valueOf(inDir)), greenwaveRealtimePO);
        }
        return realTimeMap;
    }

    /**
     * 通过绿波路口获取海信路口协调方向相位绿灯信息
     *
     * @return
     * @throws JsonProcessingException
     */
    private List<SchemeGreenDirGreenTimeDTO> getSchemeGreenDirGreenTimeDTOS(List<GreenCrossDirDTO> greenCrossDirCache) throws Exception {
        List<SchemeGreenDirGreenTimeDTO> schemeGreenDirGreenTimeDTOS = new ArrayList<>();
        ObjectMapper mapper = JacksonUtils.getInstance();

        if (!CollectionUtils.isEmpty(greenCrossDirCache)) {
            for (GreenCrossDirDTO greenCrossDirDTO : greenCrossDirCache) {
                Integer greenId = greenCrossDirDTO.getGreenId();
                String greenCrossId = greenCrossDirDTO.getCrossId();
                String greenDirStr = greenCrossDirDTO.getGreenDir();

                // 海信环相位信息
                String str = (String) redisUtils.getHash("utc-hisense-ring-phase", greenCrossId);
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                List<RingPhaseInfoDTO> ringPhaseInfoDTOS = mapper.readValue(str, new TypeReference<List<RingPhaseInfoDTO>>() {
                });
                // 遍历绿波路口协调方向
                if (StringUtils.isNotBlank(greenDirStr)) {
                    String[] greenDirArray = greenDirStr.split(",");
                    for (String s : greenDirArray) {
                        // 协调方向
                        Integer greenDir = Integer.valueOf(s);

                        if (!CollectionUtils.isEmpty(ringPhaseInfoDTOS)) {
                            Map<Integer, List<RingPhaseInfoDTO>> ringMap = ringPhaseInfoDTOS.stream().collect(Collectors.groupingBy(RingPhaseInfoDTO::getRingNo));
                            for (Map.Entry<Integer, List<RingPhaseInfoDTO>> entry : ringMap.entrySet()) {
                                Integer ringNo = entry.getKey();
                                if (Objects.equals(0, ringNo)) {
                                    continue;
                                }
                                List<RingPhaseInfoDTO> ringPhaseList = entry.getValue();
                                List<RingPhaseInfoDTO> sort = ringPhaseList.stream().sorted(Comparator.comparingInt(RingPhaseInfoDTO::getSort)).collect(Collectors.toList());
                                Integer start = 0;
                                SchemeGreenDirGreenTimeDTO schemeGreenDirGreenTimeDTO = new SchemeGreenDirGreenTimeDTO();
                                for (RingPhaseInfoDTO ringPhaseInfoDTO : sort) {
                                    List<RingPhaseInfoDTO.DirTurn> dirTurnInfoList = ringPhaseInfoDTO.getDirTurnInfoList();
                                    if (CollectionUtils.isEmpty(dirTurnInfoList)) {
                                        continue;
                                    }
                                    Integer phaseId = ringPhaseInfoDTO.getPhaseId();
                                    Integer greenTime = ringPhaseInfoDTO.getGreenTime();
                                    Integer yellowTime = ringPhaseInfoDTO.getYellowTime();
                                    Integer redTime = ringPhaseInfoDTO.getRedTime();
                                    Integer schemeNo = ringPhaseInfoDTO.getSchemeNo();
                                    Integer cycleLen = ringPhaseInfoDTO.getCycleLen();
                                    Integer offset = ringPhaseInfoDTO.getOffset();
                                    for (RingPhaseInfoDTO.DirTurn dirTurn : dirTurnInfoList) {
                                        Integer dir = dirTurn.getDir();
                                        Integer turn = dirTurn.getTurn();
                                        // 如果路口信息与绿波协调方向相同
                                        // 如果是红灯，作为开始时间，结束时间是start + greenTime + yellowTime
                                        if (Objects.equals(dir, greenDir) && Objects.equals(2, turn)) {
                                            // 黄灯做绿灯处理
                                            schemeGreenDirGreenTimeDTO.setCrossId(greenCrossId);
                                            schemeGreenDirGreenTimeDTO.setSchemeNo(schemeNo);
                                            schemeGreenDirGreenTimeDTO.setDir(dir);
                                            schemeGreenDirGreenTimeDTO.setPhaseId(phaseId);
                                            schemeGreenDirGreenTimeDTO.setStart(start);
                                            schemeGreenDirGreenTimeDTO.setRingNo(ringNo);
                                            schemeGreenDirGreenTimeDTO.setCycleLen(cycleLen);
                                            schemeGreenDirGreenTimeDTO.setOffset(offset);
                                            Integer endTime = start + greenTime + yellowTime;
                                            schemeGreenDirGreenTimeDTO.setEnd(endTime);
                                            schemeGreenDirGreenTimeDTOS.add(schemeGreenDirGreenTimeDTO);
                                        } else { // 如果是非协调方向， 相位时间作为开始时间
                                            start += greenTime + yellowTime + redTime;
                                        }
                                        log.error("当前路口：{}， 相位：{}， 协调方向：{}，开始：{}， 结束：{}", greenCrossId, phaseId, greenDir, start, schemeGreenDirGreenTimeDTO.getEnd());
                                    }
                                }
                            }
                        }
                    }
                }
            }
            log.error("路口绿灯协调方向时间：{}", mapper.writeValueAsString(schemeGreenDirGreenTimeDTOS));
        }
        return schemeGreenDirGreenTimeDTOS;
    }

}
