package net.wanji.opt.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.LocalDateTimeUtil;
import net.wanji.opt.dao.mapper.GreenwaveHistoryMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.dao.mapper.report.AnalysisGreenWaveOptimizeWeekMapper;
import net.wanji.opt.dto.report.GreenHistIndexDTO;
import net.wanji.opt.dto.report.StrategyGreenOptHistDTO;
import net.wanji.opt.entity.report.AnalysisGreenWaveOptimizeWeek;
import net.wanji.opt.service.GreenwaveHistProvider;
import org.joda.time.DateTime;
import org.joda.time.Seconds;
import org.joda.time.format.DateTimeFormat;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author fengyi
 * @date 2025/4/9
 * @description
 */
@Component
@Configurable
@EnableScheduling
@Slf4j
@Profile("!dev")
public class OptimizeStatisticTask {


    @Resource
    StrategyGreenOptHistMapper strategyGreenOptHistMapper;

    @Resource
    GreenwaveHistProvider greenwaveHistProvider;
    @Resource
    AnalysisGreenWaveOptimizeWeekMapper analysisGreenWaveOptimizeWeekMapper;
 

    /**
     * 干线策略优化时长计算
     */
    @Scheduled(cron = "0 40 1 * * ?")
    //@PostConstruct
    public void optimizeGreenWaveDurationCal() {
        long st = System.currentTimeMillis();
        try {

            List<AnalysisGreenWaveOptimizeWeek> durationList = new ArrayList<>();

            List<StrategyGreenOptHistDTO> list = strategyGreenOptHistMapper.findGreenWaveOptHist();
            Map<Integer, List<StrategyGreenOptHistDTO>> groupByGreenId = list.stream().collect(Collectors.groupingBy(o -> o.getGreenId()));

            for (Map.Entry<Integer, List<StrategyGreenOptHistDTO>> entry : groupByGreenId.entrySet()) {

                List<StrategyGreenOptHistDTO> value = entry.getValue();
                value = value.stream().sorted(Comparator.comparing(StrategyGreenOptHistDTO::getControlTime)).collect(Collectors.toList());
                int i = 0;
                for (StrategyGreenOptHistDTO result : value) {
                    if (result.getControlMethod() == -1){
                        i++;
                        continue;
                    }
//                    if (i==4){
//                        System.out.println();
//                    }
                    int seconds = 900;
                    if (i != value.size() - 1) {
                        if (value.get(i + 1).getControlMethod() == -1) {
                            //下一条记录为取消状态，当前取消时间-上次下方时间
                            seconds = Seconds.secondsBetween(new DateTime(result.getControlTime()), new DateTime(value.get(i + 1).getControlTime())).getSeconds();
                            //seconds = Math.abs(seconds);
                            //超过设定的ControlDuration取ControlDuration
                            if (seconds > result.getControlDuration()) {
                                seconds = result.getControlDuration();
                            }

                           // value.get(i + 1).setRoadDirection(result.getRoadDirection());
                           //log.info("greenId={}：干线绿波方案持续时间计算，下一条记录为取消状态 开始时间：{},截止时间：{},持续时长：{}秒",
//                                    result.getGreenId(),
//                                    new DateTime(result.getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                    new DateTime(value.get(i + 1).getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                    seconds);
                        } else if(result.getControlMethod() != -1) {
                            //两条记录之前没有取消状态，取ControlDuration
                            seconds = result.getControlDuration();
                            //log.info("greenId={}：干线绿波方案持续时间计算，两条记录之前没有取消状态 开始时间：{},截止时间：{},持续时长：{}秒",
//                                    result.getGreenId(),
//                                    new DateTime(result.getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                    new DateTime(value.get(i + 1).getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                    seconds);
                        }
                    } else if (result.getControlMethod() != -1) {
                        //最后一天记录不为-1取消状态
                        seconds = Seconds.secondsBetween(new DateTime(result.getControlTime()), new DateTime(new Date())).getSeconds();
                       // seconds = Math.abs(seconds);
                        if (seconds > result.getControlDuration()) {
                            seconds = result.getControlDuration();
                        }
                        //log.info("greenId={}：干线绿波方案持续时间计算，最后一天记录不为取消状态 开始时间：{},截止时间：{},持续时长：{}秒",
//                                result.getGreenId(),
//                                new DateTime(result.getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                new DateTime(new Date()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                seconds);

                    }

                    result.setDuration(seconds);
                    //优化控制时段-开始时间
                    String controlStartTime = DateUtil.formatDate(result.getControlTime(),"yyyy-MM-dd HH:mm:ss");
                    //优化控制时段-截止时间
                    String controlEndTime = new DateTime(result.getControlTime()).plusSeconds(seconds).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));
//                    if (result.getControlTime().after(new DateTime(result.getControlTime()).plusSeconds(seconds).toDate())){
//                        System.out.println();
//                    }
                    GreenHistIndexDTO dto = greenwaveHistProvider.findGreenIndexByDateScope(result.getGreenId(),controlStartTime,controlEndTime);

                    AnalysisGreenWaveOptimizeWeek data = new AnalysisGreenWaveOptimizeWeek();
                    data.setGreenId(result.getGreenId());
                    data.setStragetyId(result.getStragetyId());
                    data.setRoadDirection(result.getRoadDirection());
                    if (dto != null) {
                        data.setCapacity(dto.getCapacity());
                        data.setDelayTime(dto.getDelayTime());
                        data.setSpeed(dto.getSpeed());
                        data.setTravelTime(dto.getTravelTime());
                        data.setStopTimes(dto.getStopTimes());
                        data.setCongestIndex(dto.getTrafficIndex());
                    }
                    data.setWeekDay(result.getWeekDay());
                    data.setOptimizeStartTime(LocalDateTime.parse(controlStartTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                    data.setOptimizeEndTime(LocalDateTime.parse(controlEndTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                    data.setOptimizeCount(1);
                    data.setOptimizeDuration(seconds);
                    data.setWeekStartTime(LocalDateTimeUtil.convertDateToLDT(result.getWeekStartDay()));
                    data.setWeekEndTime(LocalDateTimeUtil.convertDateToLDT(result.getWeekEndDay()));
                    data.setYearWeek(result.getYearWeek());
                    data.setInsertTime(LocalDateTime.now());
                    durationList.add(data);
                    i++;
                }
            }
            if (!durationList.isEmpty()) {
                analysisGreenWaveOptimizeWeekMapper.insertBatch(durationList);
            }
            long et = System.currentTimeMillis();
            log.info("批量保存干线优化数据据耗时：{}ms，size:{}", et - st, durationList.size());

        } catch (Exception e) {
            log.error("", e);
        }
    }


}
